/*
  Copyright 1999 by James Dean Palmer.
 
  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Library General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.
 
  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Please report all bugs and problems to "james@tiger-marmalade.com".  
*/

#include <FL/Fl_Browser.H>
#include <gFL/guile_fltk.h>

/*
   1.  Smob handling functions:
*/

static SCM
Smark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
Sfree (SCM widget_smob)
{
  delete (Fl_Browser *) gh_cdr(widget_smob);
  gfl_remove (widget_smob);
  return (sizeof (Fl_Browser) + sizeof (GFL_Widget));
}

static int
Sprint (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Browser *widget = (Fl_Browser *) gh_cdr(widget_smob);
  scm_puts ("#<widget Fl_Browser ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

/*
   2.  Smob definition:
*/

GFL_SUBCLASS_TYPE ("fltk-browser",
		   "The fltk-browser displays a scrolling list of text lines.",
		   fltk_browser, fltk_browser_base,
		   Smark, Sfree, Sprint);

/*
   3.  Smob Methods:
*/

GFL_PROCEDURE_5_1 
  ( fltk_browser_new,
   "fltk-browser::new",
   "Creates a new fltk-browser with the given position, size, and "
   "label string.",
   x, y, w, h, label,
   int, int, int, int, str,
   browser,
   fltk_browser )
{
  GFL_ASSERT_5 (fltk_browser_new, x, y, w, h, label);

  Fl_Browser * widget = new Fl_Browser(gh_scm2int(x), gh_scm2int(y), 
			 gh_scm2int(w), gh_scm2int(h),
			 gh_scm2newstr(label, NULL));
  SCM widget_smob;
  GFL_NEW_WIDGET_SMOB (fltk_browser, widget_smob, widget);
  return widget_smob;
}

GFL_PROCEDURE_3_0
( fltk_browser_add, 
  "fltk-browser::add", 
  "Adds a new line of text with the associated data to the end of the"
  "browser.",
  widget, text, data, 
  fltk_browser, str, int)
{
  GFL_ASSERT_3(fltk_browser_add, widget, text, data);
  Fl_Browser *wi = (Fl_Browser *) gh_cdr(widget);
  char *text_c = gh_scm2newstr(text, NULL);
  wi->add(text_c, (void *)data);
  free (text_c);
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_0
( fltk_browser_clear, 
  "fltk-browser::clear", 
  "Removes all the lines in the browser.", 
  widget, 
  fltk_browser ) 
{
  GFL_ASSERT_1(fltk_browser_clear, widget);
  Fl_Browser *wi = (Fl_Browser *) gh_cdr(widget);
  wi->clear();
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_2_0
( fltk_browser_set_column_char, 
  "fltk-browser::set-column-char", 
  "Sets the column seperator character.  Tab (\\t) is the default.", 
  widget, a, 
  fltk_browser, int)
{
  GFL_ASSERT_2(fltk_browser_set_column_char, widget, a);
  Fl_Browser *wi = (Fl_Browser *) gh_cdr(widget);
  wi->column_char(gh_scm2char(a));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
  ( fltk_browser_get_column_char, 
   "fltk-browser::get-column-char", 
   "Gets the column seperator character", 
   widget, 
   fltk_browser,
   character,
   char)
{
  GFL_ASSERT_1(fltk_browser_get_column_char, widget);
  Fl_Browser *wi = (Fl_Browser *) gh_cdr(widget);
  return gh_char2scm(wi->column_char());
}

GFL_PROCEDURE_1_1
( fltk_browser_get_column_widths, 
  "fltk-browser::get-column-widths", 
  "Gets the vector of integers that describe the width of each column in "
  "pixels.", 
  widget, 
  fltk_browser,
  column_array,
  int_array)
{
  GFL_ASSERT_1(fltk_browser_get_column_widths, widget);
  Fl_Browser *wi = (Fl_Browser *) gh_cdr(widget);
  const int *W = wi->column_widths();
  int j, i = 0;
  while (W[i] != 0) i++;
  SCM rvalue = gh_make_vector(gh_scm2int(i), gh_scm2int(0));
  for (j=0; j<i; j++)
    rvalue = gh_vector_set_x (rvalue, gh_int2scm(j), gh_int2scm(W[j]));
  return (rvalue);
}

GFL_PROCEDURE_2_0
( fltk_browser_set_column_widths, 
  "fltk-browser::set-column-widths", 
  "Sets the vector of integers that describe the width of each column in "
  "pixels.", 
  widget, a, 
  fltk_browser, int_array)
{
  GFL_ASSERT_2(fltk_browser_set_column_widths, widget, a);
  long length = gh_vector_length (a);
  Fl_Browser *wi = (Fl_Browser *) gh_cdr(widget);
  //TODO: When do I need to delete this?
  int *W = new int[length+1];
  for (int i=0; i<length; i++)
    W[i] = gh_scm2int(gh_vector_ref(a, gh_int2scm(i)));
  W[length] = 0;
  wi->column_widths(W);
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_3_0
( fltk_browser_set_data, 
  "fltk-browser::set-data", 
  "Sets the data item to be attached to the text at line_number.", 
  widget, line_number, data, 
  fltk_browser, int, int)
{
  GFL_ASSERT_3(fltk_browser_set_data, widget, line_number, data);
  Fl_Browser *wi = (Fl_Browser *) gh_cdr(widget);
  wi->data(gh_scm2int(line_number), (void *)data);
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_2_0
( fltk_browser_get_data, 
  "fltk-browser::get-data", 
  "Gets the data item attached to the text at line_number.", 
  widget, line_number, 
  fltk_browser, int)
{
  GFL_ASSERT_2(fltk_browser_get_data, widget, line_number);
  Fl_Browser *wi = (Fl_Browser *) gh_cdr(widget);
  return (SCM)wi->data(gh_scm2int(line_number));
}

GFL_PROCEDURE_2_0
( fltk_browser_set_format_char, 
  "fltk-browser::set-format-char", 
  "Sets the format code prefix character.  The default is @.", 
  widget, a, 
  fltk_browser, int)
{
  GFL_ASSERT_2(fltk_browser_set_format_char, widget, a);
  Fl_Browser *wi = (Fl_Browser *) gh_cdr(widget);
  wi->format_char(gh_scm2char(a));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_0
( fltk_browser_get_format_char, 
  "fltk-browser::get-format-char", 
  "Gets the format code prefix character.", 
  widget, 
  fltk_browser )
{
  GFL_ASSERT_1(fltk_browser_get_format_char, widget);
  Fl_Browser *wi = (Fl_Browser *) gh_cdr(widget);
  return gh_char2scm(wi->format_char());
}

GFL_PROCEDURE_2_0
( fltk_browser_hide, 
  "fltk-browser::hide", 
  "Hides the text on line_number, preventing selection by user.", 
  widget, line_number, 
  fltk_browser, int)
{
  GFL_ASSERT_2(fltk_browser_hide, widget, line_number);
  Fl_Browser *wi = (Fl_Browser *) gh_cdr(widget);
  wi->hide(gh_scm2int(line_number));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_4_0
( fltk_browser_insert, 
  "fltk-browser::insert", 
  "Inserts a newline before line_number with the  passed text and "
  "associated data.",
  widget, line_number, text, data, 
  fltk_browser, int, str, int)
{
  GFL_ASSERT_4(fltk_browser_insert, widget, line_number, text, data);
  Fl_Browser *wi = (Fl_Browser *) gh_cdr(widget);
  wi->insert(gh_scm2int(line_number), strdup(SCM_CHARS(text)), (void *)data);
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_2_1
( fltk_browser_load, 
  "fltk-browser::load", 
  "Loads the browser text lines from file_name.", 
  widget, file_name, 
  fltk_browser, str, 
  rval, 
  bool)
{
  GFL_ASSERT_2(fltk_browser_load, widget, file_name);  
  Fl_Browser *wi = (Fl_Browser *) gh_cdr(widget);
  if (wi->load(SCM_CHARS(file_name)))
    return SCM_BOOL_T;
  return SCM_BOOL_F;
}

GFL_PROCEDURE_3_0
( fltk_browser_move, 
  "fltk-browser::move", 
  "Line from is removed and inserted at line to.  to is calculated after the "
  "line is removed.", 
  widget, to, from, 
  fltk_browser, int, int) 
{
  GFL_ASSERT_3(fltk_browser_move, widget, to, from);
  Fl_Browser *wi = (Fl_Browser *) gh_cdr(widget);
  wi->move(gh_scm2int(to), gh_scm2int(from));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_2_0
( fltk_browser_set_position, 
  "fltk-browser::set-position", 
  "Sets the current vertical scrollbar position, where 0 is the top.", 
  widget, position, 
  fltk_browser, int)
{
  GFL_ASSERT_2(fltk_browser_set_position, widget, position);
  Fl_Browser *wi = (Fl_Browser *) gh_cdr(widget);
  wi->position(gh_scm2int(position));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_browser_get_position, 
  "fltk-browser::get-position", 
  "Gets the current vertical scrollbar position, where 0 is the top.", 
  widget, 
  fltk_browser, 
  position, 
  int)
{
  GFL_ASSERT_1(fltk_browser_get_position, widget);
  Fl_Browser *wi = (Fl_Browser *) gh_cdr(widget);
  return gh_int2scm(wi->position());
}

GFL_PROCEDURE_2_0
( fltk_browser_remove, 
  "fltk-browser::remove", 
  "Removes the text at line_number.", 
  widget, line_number, 
  fltk_browser, int)
{
  GFL_ASSERT_2(fltk_browser_remove, widget, line_number);
  Fl_Browser *wi = (Fl_Browser *) gh_cdr(widget);
  wi->remove(gh_scm2int(line_number));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_2_0
( fltk_browser_show, 
  "fltk-browser::show", 
  "Makes line_number visible for selection.", 
  widget, line_number, 
  fltk_browser, int)
{
  GFL_ASSERT_2(fltk_browser_show, widget, line_number);
  Fl_Browser *wi = (Fl_Browser *) gh_cdr(widget);
  wi->show(gh_scm2int(line_number));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
  ( fltk_browser_get_size, 
   "fltk-browser::get-size", 
   "docstring", 
   widget, 
   fltk_browser, 
   size, 
   int)
{
  GFL_ASSERT_1(fltk_browser_get_size, widget);
  Fl_Browser *wi = (Fl_Browser *) gh_cdr(widget);
  return gh_int2scm(wi->size());
}

GFL_PROCEDURE_3_0
( fltk_browser_set_text, 
  "fltk-browser::set-text", 
  "Sets the text at line_number.", 
  widget, line_number, text, 
  fltk_browser, int, str)
{
  GFL_ASSERT_3(fltk_browser_set_text, widget, line_number, text);
  Fl_Browser *wi = (Fl_Browser *) gh_cdr(widget);
  wi->text(gh_scm2int(line_number), strdup(SCM_CHARS(text)));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_2_0
( fltk_browser_get_text, 
  "fltk-browser::get-text", 
  "Gets the text at line_number.", 
  widget, line_number, 
  fltk_browser, str)
{
  GFL_ASSERT_2(fltk_browser_get_text, widget, line_number); 
  Fl_Browser *wi = (Fl_Browser *) gh_cdr(widget);
  return scm_makfrom0str (wi->text(gh_scm2int(line_number)));
}

GFL_PROCEDURE_2_0
( fltk_browser_set_topline, 
  "fltk-browser::set-topline", 
  "Sets the top line to line_number by scrolling the browser.",
  widget, line_number, 
  fltk_browser, int)
{
  GFL_ASSERT_2(fltk_browser_set_topline, widget, line_number);
  Fl_Browser *wi = (Fl_Browser *) gh_cdr(widget);
  wi->topline(gh_scm2int(line_number));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_browser_get_topline, 
  "fltk-browser::get-topline", 
  "Gets the top line line_number.", 
  widget, 
  fltk_browser, 
  rval, 
  int)
{
  GFL_ASSERT_1(fltk_browser_get_topline, widget);  
  Fl_Browser *wi = (Fl_Browser *) gh_cdr(widget);
  return gh_int2scm(wi->topline());
}

GFL_PROCEDURE_2_1
( fltk_browser_visible, 
  "fltk-browser::visible", 
  "Returns true if line line_number is visible.", 
  widget, line_number, 
  fltk_browser, int, 
  rval, 
  bool)
{
  GFL_ASSERT_2(fltk_browser_visible, widget, line_number);
  Fl_Browser *wi = (Fl_Browser *) gh_cdr(widget);
  if (wi->visible(gh_scm2int(line_number)))
    return SCM_BOOL_T;
  return SCM_BOOL_F;
}

GFL_PROCEDURE_2_0
( fltk_browser_set_middleline, 
  "fltk-browser::set-middleline", 
  "Scrolls the browser so the middle line in the browser is line_number.",
  widget, line_number, 
  fltk_browser, int )
{
  GFL_ASSERT_2(fltk_browser_visible, widget, line_number);
  Fl_Browser *wi = (Fl_Browser *) gh_cdr(widget);
  wi->middleline(gh_scm2int(line_number));
  return SCM_UNSPECIFIED;
}

/*
   4.  Initialization Hook:
*/

void
fltk_browser_init ()
{
  GFL_NEW_TYPE (fltk_browser);
  GFL_NEW_PROCEDURE (fltk_browser_new);
  GFL_NEW_PROCEDURE (fltk_browser_add);
  GFL_NEW_PROCEDURE (fltk_browser_clear);
  GFL_NEW_PROCEDURE (fltk_browser_set_column_char);
  GFL_NEW_PROCEDURE (fltk_browser_get_column_char);
  GFL_NEW_PROCEDURE (fltk_browser_get_column_widths);
  GFL_NEW_PROCEDURE (fltk_browser_set_column_widths);
  GFL_NEW_PROCEDURE (fltk_browser_set_data);
  GFL_NEW_PROCEDURE (fltk_browser_get_data);
  GFL_NEW_PROCEDURE (fltk_browser_set_format_char);
  GFL_NEW_PROCEDURE (fltk_browser_get_format_char);
  GFL_NEW_PROCEDURE (fltk_browser_hide);
  GFL_NEW_PROCEDURE (fltk_browser_insert);
  GFL_NEW_PROCEDURE (fltk_browser_load);
  GFL_NEW_PROCEDURE (fltk_browser_move);
  GFL_NEW_PROCEDURE (fltk_browser_set_position);
  GFL_NEW_PROCEDURE (fltk_browser_get_position);
  GFL_NEW_PROCEDURE (fltk_browser_remove);
  GFL_NEW_PROCEDURE (fltk_browser_show);
  GFL_NEW_PROCEDURE (fltk_browser_get_size);
  GFL_NEW_PROCEDURE (fltk_browser_set_text);
  GFL_NEW_PROCEDURE (fltk_browser_get_text);
  GFL_NEW_PROCEDURE (fltk_browser_get_topline);
  GFL_NEW_PROCEDURE (fltk_browser_set_topline);
  GFL_NEW_PROCEDURE (fltk_browser_set_middleline);
  GFL_NEW_PROCEDURE (fltk_browser_visible);

  gh_define("FL_NORMAL_BROWSER", gh_long2scm(FL_NORMAL_BROWSER));
  gh_define("FL_SELECT_BROWSER", gh_long2scm(FL_SELECT_BROWSER));
  gh_define("FL_HOLD_BROWSER", gh_long2scm(FL_HOLD_BROWSER));
  gh_define("FL_MULTI_BROWSER", gh_long2scm(FL_MULTI_BROWSER));
}
