/*
  Copyright 1999 by James Dean Palmer.
 
  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Library General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.
 
  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Please report all bugs and problems to "james@tiger-marmalade.com".  
*/

#include <FL/Fl_Group.H>
#include <gFL/guile_fltk.h>

/*
   1.  Smob handling functions:
*/

static SCM
Smark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
Sfree (SCM widget_smob)
{
  delete (Fl_Group *) gh_cdr (widget_smob);
  gfl_remove (widget_smob);
  return (sizeof (Fl_Group) + sizeof (Fl_Group));
}

static int
Sprint (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Group *widget = (Fl_Group *) gh_cdr (widget_smob);
  scm_puts ("#<widget Fl_Group ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

/*
   2.  Smob definition:
*/

GFL_SUBCLASS_TYPE ("fltk-group",
		   "fltk-group is the fltk container widget.  It maintains a list of child widgets.",
		   fltk_group, fltk_widget,
		   Smark, Sfree, Sprint);

/*
   3.  Smob Methods:
*/

GFL_PROCEDURE_5_1 
( fltk_group_new,
  "fltk-group::new",
  "Creates a new fltk-group with the given position, size, and "
  "label string.",
  x, y, w, h, label,
  int, int, int, int, str,
  group,
  fltk_group )
{
  GFL_ASSERT_5 (fltk_group_new, x, y, w, h, label);
  
  Fl_Group * widget = new Fl_Group(gh_scm2int(x), gh_scm2int(y), 
				   gh_scm2int(w), gh_scm2int(h),
				   gh_scm2newstr(label, NULL));
  SCM widget_smob;
  GFL_NEW_WIDGET_SMOB (fltk_group, widget_smob, widget);
  return widget_smob;
}

GFL_PROCEDURE_1_1
( fltk_group_children, 
  "fltk-group::children", 
  "Returns the number of child widgets the group has.", 
  widget, 
  fltk_group, 
  count, 
  int)
{
  GFL_ASSERT_1(fltk_group_children, widget);
  Fl_Group *wi = (Fl_Group *) gh_cdr (widget);
  return gh_int2scm((int)wi->children());
}

GFL_PROCEDURE_2_1
( fltk_group_child, 
  "fltk-group::child", 
  "Returns the Nth widget in this group.", 
  widget, N, 
  fltk_group, int, 
  rval, 
  fltk_group)
{
  GFL_ASSERT_2(fltk_group_child, widget, N);
  Fl_Group *wi = (Fl_Group *) gh_cdr (widget);
  return gfl_lookup((Fl_Widget *) wi->child(gh_scm2int(N)));
}

GFL_PROCEDURE_2_0
( fltk_group_add, 
  "fltk-group::add", 
  "The child widget is removed from it's group and then added to this group.", 
  widget, child, 
  fltk_group, fltk_widget)
{
  GFL_ASSERT_2(fltk_group_add, widget, child);
  Fl_Group *wi = (Fl_Group *) gh_cdr (widget);
  wi->add((Fl_Widget *) gh_cdr (child));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_2_0
( fltk_group_add_resizable, 
  "fltk-group::add-resizable", 
  "Adds a child widget to the group and makes it the resizeable widget.", 
  widget, child, 
  fltk_group, fltk_widget)
{
  GFL_ASSERT_2(fltk_group_add_resizable, widget, child);
  Fl_Group *wi = (Fl_Group *) gh_cdr (widget);
  wi->add_resizable(*(Fl_Widget *) gh_cdr (child));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_0
( fltk_group_begin, 
  "fltk-group::begin", 
  "Sets this group as the current group that other widgets will be added to.  Don't forget to end the group!", 
  widget, 
 fltk_group )
{
  GFL_ASSERT_1(fltk_group_begin, widget);
  Fl_Group *wi = (Fl_Group *) gh_cdr (widget);
  wi->begin();
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_0
( fltk_group_end, 
  "fltk-group::end", 
  "Ends this group, and sets the current group to it's parent.", 
  widget, 
 fltk_group )
{
  GFL_ASSERT_1(fltk_group_end, widget);
  Fl_Group *wi = (Fl_Group *) gh_cdr (widget);
  wi->end();
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_0
( fltk_group_set_current, 
  "fltk-group::set-current", 
  "Sets the currently active group", 
  widget, 
 fltk_group )
{
  GFL_ASSERT_1(fltk_group_set_current, widget);
  Fl_Group::current( (Fl_Group *) gh_cdr (widget));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_0_1
( fltk_group_get_current, 
  "fltk-group::get-current", 
  "Gets the currently active group.", 
  widget, 
  fltk_group)
{
  return gfl_lookup((Fl_Widget *) Fl_Group::current());
}

GFL_PROCEDURE_2_1
( fltk_group_find, 
  "fltk-group::find", 
  "Searches the child array for the widget and returns the index.", 
  widget, a, 
  fltk_group, fltk_widget, 
  rval, 
  int)
{
  GFL_ASSERT_2(fltk_group_find, widget, a);
  Fl_Group *wi = (Fl_Group *) gh_cdr (widget);
  return gh_int2scm(wi->find((Fl_Widget *) gh_cdr (a)));
}

GFL_PROCEDURE_3_0
( fltk_group_insert, 
  "fltk-group::insert", 
  "The child widget is removed from it's current group and then inserted into this group before index N.", 
  widget, child, N, 
  fltk_group, fltk_widget, int)
{
  GFL_ASSERT_3(fltk_group_insert, widget, child, N);
  Fl_Group *wi = (Fl_Group *) gh_cdr (widget);
  wi->insert(*(Fl_Widget *) gh_cdr (child), gh_scm2int(N));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_2_0
( fltk_group_remove, 
  "fltk-group::remove", 
  "Removes a child widget from the group.", 
  widget, child, 
  fltk_group, fltk_widget)
{
  GFL_ASSERT_2(fltk_group_remove, widget, child);
  Fl_Group *wi = (Fl_Group *) gh_cdr (widget);
  wi->remove((Fl_Widget *) gh_cdr (child));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_0
( fltk_group_clear, 
  "fltk-group::clear", 
  "Removes all children from the group.", 
  widget, 
 fltk_group )
{
  GFL_ASSERT_1(fltk_group_clear, widget);
  Fl_Group *wi = (Fl_Group *) gh_cdr (widget);
  wi->clear();
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_2_0
( fltk_group_set_resizable, 
  "fltk-group::set-resizable", 
  "Sets the resizeable child widget.",
  widget, a, 
  fltk_group, fltk_widget)
{
  GFL_ASSERT_2(fltk_group_set_resizable, widget, a);
  Fl_Group *wi = (Fl_Group *) gh_cdr (widget);
  wi->resizable( (Fl_Widget *) gh_cdr (a));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_group_get_resizable, 
  "fltk-group::get-resizable", 
  "Gets the resizaable child widget.", 
  widget, 
  fltk_group, 
  rval, 
  fltk_group)
{
  GFL_ASSERT_1(fltk_group_get_resizable, widget);
  Fl_Group *wi = (Fl_Group *) gh_cdr (widget);
  return gfl_lookup((Fl_Widget *) wi->resizable());
}

/*
   4.  Initialization Hook:
*/

void
fltk_group_init ()
{
  GFL_NEW_TYPE (fltk_group);
  GFL_NEW_PROCEDURE (fltk_group_new);
  GFL_NEW_PROCEDURE (fltk_group_children);
  GFL_NEW_PROCEDURE (fltk_group_child);
  GFL_NEW_PROCEDURE (fltk_group_add);
  GFL_NEW_PROCEDURE (fltk_group_add_resizable);
  GFL_NEW_PROCEDURE (fltk_group_begin);
  GFL_NEW_PROCEDURE (fltk_group_end);
  GFL_NEW_PROCEDURE (fltk_group_set_current);
  GFL_NEW_PROCEDURE (fltk_group_get_current);
  GFL_NEW_PROCEDURE (fltk_group_find);
  GFL_NEW_PROCEDURE (fltk_group_insert);
  GFL_NEW_PROCEDURE (fltk_group_remove);
  GFL_NEW_PROCEDURE (fltk_group_clear);
  GFL_NEW_PROCEDURE (fltk_group_set_resizable);
  GFL_NEW_PROCEDURE (fltk_group_get_resizable);
}
