/*
  Copyright 1999 by James Dean Palmer.
 
  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Library General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.
 
  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Please report all bugs and problems to "james@tiger-marmalade.com".  
*/

#include <FL/Fl_Menu_.H>
#include <gFL/guile_fltk.h>

/*
   1.  Smob handling functions:
*/

static SCM
Smark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
Sfree (SCM widget_smob)
{
  delete (Fl_Menu_ *) gh_cdr (widget_smob);
  gfl_remove (widget_smob);
  return (sizeof (GFL_Widget) + sizeof (Fl_Menu_));
}

static int
Sprint (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Menu_ *widget = (Fl_Menu_ *) gh_cdr (widget_smob);
  scm_puts ("#<widget Fl_Menu_ ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

/*
   2.  Smob definition:
*/

GFL_SUBCLASS_TYPE ("fltk-menu", 
		   "fltk-menu is the base class for all widgets that provide menus.",
		   fltk_menu, fltk_widget, 
		   Smark, Sfree, Sprint);

/*
   3.  Smob Methods:
*/

GFL_PROCEDURE_2_0
( fltk_menu_set_value, 
  "fltk-menu::set-value", 
  "Sets the last selected menu item index.", 
  widget, a, 
  fltk_menu, int)
{
  GFL_ASSERT_2(fltk_menu_set_value, widget, a);
  Fl_Menu_ *wi = (Fl_Menu_ *) gh_cdr (widget);
  wi->value(gh_scm2int(a));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_menu_get_value, 
  "fltk-menu::get-value", 
  "Gets the last selected menu item index.", 
  widget, 
  fltk_menu, 
  rval, 
  int)
{
  GFL_ASSERT_1(fltk_menu_get_value, widget); 
  Fl_Menu_ *wi = (Fl_Menu_ *) gh_cdr (widget);
  return gh_int2scm((int)wi->value());
}

GFL_PROCEDURE_1_0
( fltk_menu_global, 
  "fltk-menu::global", 
  "Makes the shortcuts for this menu \"global\", that is they will be trapped no matter what window is the focus.", 
  widget, 
  fltk_menu)
{
  GFL_ASSERT_1(fltk_menu_global, widget);
  Fl_Menu_ *wi = (Fl_Menu_ *) gh_cdr (widget);
  wi->global();
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_2_0
( fltk_menu_text, 
  "fltk-menu::text", 
  "Returns the title of the Nth menu item.", 
  widget, N, 
  fltk_menu, int) 
{
  GFL_ASSERT_2(fltk_menu_text, widget, N);
  Fl_Menu_ *wi = (Fl_Menu_ *) gh_cdr (widget);
  return scm_makfrom0str (wi->text(gh_scm2int(N)));
}

GFL_PROCEDURE_1_1
( fltk_menu_size, 
  "fltk-menu::size", 
  "Returns the number of entries in the menu.", 
  widget, 
  fltk_menu, 
  rval, 
  int)
{
  GFL_ASSERT_1(fltk_menu_size, widget);
  Fl_Menu_ *wi = (Fl_Menu_ *) gh_cdr (widget);
  return gh_int2scm((int)wi->size());
}

GFL_PROCEDURE_5_0
( fltk_menu_add, 
  "fltk-menu::add", 
  "Adds a new menu item with the given title, shortcut, callback and flags.", 
  widget, title, shortcut, callback, flags, 
  fltk_menu, str, str, callback, int)
{
  GFL_ASSERT_5(fltk_menu_add, widget, title, shortcut, callback, flags);
  Fl_Menu_ *wi = (Fl_Menu_ *) gh_cdr (widget);
  if (gh_boolean_p(callback))
    wi->add(gh_scm2newstr(title, NULL), 
	    gh_scm2newstr(shortcut, NULL), 
	    0, 0, gh_scm2int(flags));
  else
    {
      scm_protect_object (callback);
      wi->add(gh_scm2newstr(title, NULL), 
	      gh_scm2newstr(shortcut, NULL), 
	      guile_fltk_callback2, 
	      (void *)callback, gh_scm2int(flags));
    }
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_0
( fltk_menu_clear, 
  "fltk-menu::clear", 
  "Removes all menu items from the menu.", 
  widget, 
  fltk_menu )
{
  GFL_ASSERT_1(fltk_menu_clear, widget);
  Fl_Menu_ *wi = (Fl_Menu_ *) gh_cdr (widget);
  wi->clear();
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_3_0
( fltk_menu_replace, 
  "fltk-menu::replace", 
  "Replaces Nth menu item's text.", 
  widget, N, text, 
  fltk_menu, int, str) 
{
  GFL_ASSERT_3(fltk_menu_replace, widget, N, text);
  Fl_Menu_ *wi = (Fl_Menu_ *) gh_cdr (widget);
  wi->replace(gh_scm2int(N), gh_scm2newstr(text, NULL));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_2_0
( fltk_menu_remove, 
  "fltk-menu::remove", 
  "Remove the Nth menu item.", 
  widget, N, 
  fltk_menu, int)
{
  GFL_ASSERT_2(fltk_menu_remove, widget, N);
  Fl_Menu_ *wi = (Fl_Menu_ *) gh_cdr (widget);
  wi->remove(gh_scm2int(N));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_3_0
( fltk_menu_shortcut, 
  "fltk-menu::shortcut", 
  "Sets the shortcut of the Nth menu item.", 
  widget, N, shortcut, 
  fltk_menu, int, int)
{
  GFL_ASSERT_3(fltk_menu_shortcut, widget, N, shortcut);
  Fl_Menu_ *wi = (Fl_Menu_ *) gh_cdr (widget);
  wi->shortcut(gh_scm2int(N), gh_scm2int(shortcut));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_3_0
( fltk_menu_mode, 
  "fltk-menu::mode", 
  "Changes the flags of the Nth menu item.", 
  widget, N, flags, 
  fltk_menu, int, int)
{
  GFL_ASSERT_3(fltk_menu_mode, widget, N, flags);
  Fl_Menu_ *wi = (Fl_Menu_ *) gh_cdr (widget);
  wi->mode(gh_scm2int(N), gh_scm2int(flags));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_2_0
( fltk_menu_set_text_color, 
  "fltk-menu::set-text-color", 
  "Sets the text color.", 
  widget, color, 
  fltk_menu, int)
{
  GFL_ASSERT_2(fltk_menu_set_text_color, widget, color);
  Fl_Menu_ *wi = (Fl_Menu_ *) gh_cdr (widget);
  wi->textcolor(gh_scm2int(color));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_menu_get_text_color, 
  "fltk-menu::get-text-color", 
  "Gets the text color.", 
  widget, 
  fltk_menu, 
  rval, 
  int)
{
  GFL_ASSERT_1(fltk_menu_get_text_color, widget);
  Fl_Menu_ *wi = (Fl_Menu_ *) gh_cdr (widget);
  return gh_int2scm((int)wi->textcolor());
}

GFL_PROCEDURE_2_0
( fltk_menu_set_text_font, 
  "fltk-menu::set-text-font", 
  "Sets the text font.", 
  widget, a, 
  fltk_menu, int)
{
  GFL_ASSERT_2(fltk_menu_set_text_font, widget, a); 
  Fl_Menu_ *wi = (Fl_Menu_ *) gh_cdr (widget);
  wi->textfont(gh_scm2int(a));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_menu_get_text_font, 
  "fltk-menu::get-text-font", 
  "Gets the text font.", 
  widget, 
  fltk_menu, 
  rval, 
  int)
{
  GFL_ASSERT_1(fltk_menu_get_text_font, widget); 
  Fl_Menu_ *wi = (Fl_Menu_ *) gh_cdr (widget);
  return gh_int2scm((int)wi->textfont());
}

GFL_PROCEDURE_2_0
( fltk_menu_set_text_size, 
  "fltk-menu::set-text-size", 
  "Sets the text font size.", 
  widget, a, 
  fltk_menu, int)
{
  GFL_ASSERT_2(fltk_menu_set_text_size, widget, a); 
  Fl_Menu_ *wi = (Fl_Menu_ *) gh_cdr (widget);
  wi->textsize(gh_scm2int(a));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_menu_get_text_size, 
  "fltk-menu::get-textsize", 
  "Gets the text font size.", 
  widget, 
  fltk_menu, 
  rval, 
  int)
{
  GFL_ASSERT_1(fltk_menu_get_text_size, widget);
  Fl_Menu_ *wi = (Fl_Menu_ *) gh_cdr (widget);
  return gh_int2scm((int)wi->textsize());
}

GFL_PROCEDURE_2_0
( fltk_menu_set_down_box, 
  "fltk-menu::set-down-box", 
  "Sets the boxtype that is drawn when the menu is selected.", 
  widget, a, 
  fltk_menu, int)
{
  GFL_ASSERT_2(fltk_menu_set_down_box, widget, a);
  Fl_Menu_ *wi = (Fl_Menu_ *) gh_cdr (widget);
  wi->down_box((Fl_Boxtype)gh_scm2int(a));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_1_1
( fltk_menu_get_down_box, 
  "fltk-menu::get-down-box", 
  "Gets the boxtype that is drawn when the menu is selected.", 
  widget, 
  fltk_menu, 
  rval, 
  int)
{
  GFL_ASSERT_1(fltk_menu_get_down_box, widget);
  Fl_Menu_ *wi = (Fl_Menu_ *) gh_cdr (widget);
  return gh_int2scm((int)wi->down_box());
}

/*
   4.  Initialization Hook:
*/

void
fltk_menu_init ()
{
  GFL_NEW_TYPE (fltk_menu);
  GFL_NEW_PROCEDURE (fltk_menu_set_value);
  GFL_NEW_PROCEDURE (fltk_menu_get_value);
  GFL_NEW_PROCEDURE (fltk_menu_global);
  GFL_NEW_PROCEDURE (fltk_menu_text);
  GFL_NEW_PROCEDURE (fltk_menu_size);
  GFL_NEW_PROCEDURE (fltk_menu_add);
  GFL_NEW_PROCEDURE (fltk_menu_clear);
  GFL_NEW_PROCEDURE (fltk_menu_replace);
  GFL_NEW_PROCEDURE (fltk_menu_remove);
  GFL_NEW_PROCEDURE (fltk_menu_shortcut);
  GFL_NEW_PROCEDURE (fltk_menu_mode);
  GFL_NEW_PROCEDURE (fltk_menu_set_text_color);
  GFL_NEW_PROCEDURE (fltk_menu_get_text_color);
  GFL_NEW_PROCEDURE (fltk_menu_set_text_font);
  GFL_NEW_PROCEDURE (fltk_menu_get_text_font);
  GFL_NEW_PROCEDURE (fltk_menu_set_text_size);
  GFL_NEW_PROCEDURE (fltk_menu_get_text_size);
  GFL_NEW_PROCEDURE (fltk_menu_set_down_box);
  GFL_NEW_PROCEDURE (fltk_menu_get_down_box);

  gh_define ("FL_MENU_INACTIVE", gh_int2scm(1));
  gh_define ("FL_MENU_TOGGLE", gh_int2scm(2));
  gh_define ("FL_MENU_VALUE", gh_int2scm(4));
  gh_define ("FL_MENU_RADIO", gh_int2scm(8));
  gh_define ("FL_MENU_INVISIBLE", gh_int2scm(16));
  gh_define ("FL_SUBMENU_POINTER", gh_int2scm(32));
  gh_define ("FL_SUBMENU", gh_int2scm(64));
  gh_define ("FL_MENU_DIVIDER", gh_int2scm(128));
  gh_define ("FL_MENU_HORIZONTAL", gh_int2scm(256));
}
