/*
  Copyright 1999 by James Dean Palmer.
 
  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Library General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.
 
  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Please report all bugs and problems to "james@tiger-marmalade.com".  
*/

#include <FL/Fl_Tile.H>
#include <gFL/guile_fltk.h>
GFL_EXTERN_TYPE (fltk_widget);

/*
   1.  Smob handling functions:
*/

static SCM
Smark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
Sfree (SCM widget_smob)
{
  delete (Fl_Tile *) gh_cdr (widget_smob);
  gfl_remove (widget_smob);
  return (sizeof (Fl_Tile) + sizeof (Fl_Tile));
}

static int
Sprint (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Tile *widget = (Fl_Tile *) gh_cdr (widget_smob);
  scm_puts ("#<widget Fl_Tile ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

/*
   2.  Smob definition:
*/

GFL_SUBCLASS_TYPE ("fltk-tile", 
		   "fltk-tile is a fltk-group subclass that lets you resize "
		   "the children by dragging the border between them.",
		   fltk_tile, fltk_group, 
		   Smark, Sfree, Sprint);

/*
   3.  Smob Methods:
*/

GFL_PROCEDURE_5_1 
( fltk_tile_new,
  "fltk-tile::new",
  "Creates a new fltk-tile with the given position, size, and "
  "label string.",
  x, y, w, h, label,
  int, int, int, int, str,
  tile,
  fltk_tile )
{
  GFL_ASSERT_5 (fltk_tile_new, x, y, w, h, label);
  Fl_Tile * widget = new Fl_Tile(gh_scm2int(x), gh_scm2int(y), 
			 gh_scm2int(w), gh_scm2int(h),
			 gh_scm2newstr(label, NULL));
  SCM widget_smob;
  GFL_NEW_WIDGET_SMOB (fltk_tile, widget_smob, widget);
  return widget_smob;
}

GFL_PROCEDURE_5_0
  ( fltk_tile_position, 
   "fltk-tile::position", 
   "Drag the intersection at from_x,from_y to to_x,to_y. This redraws "
    "all the necessary children.", 
   widget, from_x, from_y, to_x, to_y, 
   fltk_tile, int, int, int, int)
{
  GFL_ASSERT_5(fltk_tile_position, widget, from_x, from_y, to_x, to_y);
  Fl_Tile *wi = (Fl_Tile *) gh_cdr (widget);
  wi->position(gh_scm2int(from_x), gh_scm2int(from_y), 
	       gh_scm2int(to_x), gh_scm2int(to_y));
  return SCM_UNSPECIFIED;
}

GFL_PROCEDURE_5_0
( fltk_tile_resize, 
  "fltk-tile::resize", 
  "Resizes the window to the given position and size.", 
  widget, x, y, w, h, 
  fltk_tile, int, int, int, int)
{
  GFL_ASSERT_5(fltk_tile_resize, widget, x, y, w, h);
  Fl_Tile *wi = (Fl_Tile *) gh_cdr (widget);
  wi->resize(gh_scm2int(x), gh_scm2int(y), 
	     gh_scm2int(w), gh_scm2int(h));
  return SCM_UNSPECIFIED;
}

// The next few functions are "extra" convenience functions that have 
// no fltk analog.  They could easily be added to the Fl_Tile class or
// a subclass.

static void
tile_insert_right (Fl_Tile *t, Fl_Widget *u, Fl_Widget *i)
{
  int dw = u->w() / 2;
  u->size (u->w() - dw, u->h());
  i->size (dw, u->h());
  i->position (u->x() + u->w(), u->y());
  t->add (i);
}

GFL_PROCEDURE_3_0
  (fltk_tile_insert_right,
   "fltk-tile::insert-right",
   "Divides X in half vertically, and inserts Y in the right half.",
   tile, X, Y,
   fltk_tile, fltk_widget, fltk_widget )
{
  GFL_ASSERT_3 (fltk_tile_insert_right, tile, X, Y);
  tile_insert_right ((Fl_Tile *) gh_cdr (tile), (Fl_Widget *) gh_cdr (X),
		     (Fl_Widget *) gh_cdr (Y));
  return SCM_UNSPECIFIED;
}

static void
tile_insert_left (Fl_Tile *t, Fl_Widget *u, Fl_Widget *i)
{
  int dw = u->w() / 2;
  u->size (u->w() - dw, u->h());
  i->size (dw, u->h());
  i->position (u->x(), u->y());
  u->position (i->x() + i->w(), u->y());
  t->add (i);
}

GFL_PROCEDURE_3_0
  (fltk_tile_insert_left,
   "fltk-tile::insert-left",
   "Divides X in half vertically, and inserts Y in the left half.",
   tile, X, Y,
   fltk_tile, fltk_widget, fltk_widget )
{
  GFL_ASSERT_3 (fltk_tile_insert_left, tile, X, Y);
  tile_insert_left ((Fl_Tile *) gh_cdr (tile), (Fl_Widget *) gh_cdr (X),
		     (Fl_Widget *) gh_cdr (Y));
  return SCM_UNSPECIFIED;
}

static void
tile_insert_down (Fl_Tile *t, Fl_Widget *u, Fl_Widget *i)
{
  int dh = u->h() / 2;
  u->size (u->w(), u->h() - dh);
  i->size (u->w(), dh);
  i->position (u->x(), u->y() + u->h());
  t->add (i);
}

GFL_PROCEDURE_3_0
  (fltk_tile_insert_down,
   "fltk-tile::insert-down",
   "Divides X in half horizontally, and inserts Y in the lower half.",
   tile, X, Y,
   fltk_tile, fltk_widget, fltk_widget )
{
  GFL_ASSERT_3 (fltk_tile_insert_down, tile, X, Y);
  tile_insert_down ((Fl_Tile *) gh_cdr (tile), (Fl_Widget *) gh_cdr (X),
		    (Fl_Widget *) gh_cdr (Y));
  return SCM_UNSPECIFIED;
}

static void
tile_insert_up (Fl_Tile *t, Fl_Widget *u, Fl_Widget *i)
{
  int dh = u->h() / 2;
  u->size (u->w(), u->h() - dh);
  i->size (u->w(), dh);
  i->position (u->x(), u->y());
  u->position (i->x(), i->y() + i->h());
  t->add (i);
}

GFL_PROCEDURE_3_0
  (fltk_tile_insert_up,
   "fltk-tile::insert-up",
   "Divides X in half horizontally, and inserts Y in the upper half.",
   tile, X, Y,
   fltk_tile, fltk_widget, fltk_widget )
{
  GFL_ASSERT_3 (fltk_tile_insert_up, tile, X, Y);
  tile_insert_up ((Fl_Tile *) gh_cdr (tile), (Fl_Widget *) gh_cdr (X),
		     (Fl_Widget *) gh_cdr (Y));
  return SCM_UNSPECIFIED;
}

static Fl_Widget*
tile_fill_right (Fl_Tile *t, Fl_Widget *u)
{
  Fl_Widget*const* a = t->array();
  int dw; // number of pixels to add to u's width.
  for (int i=t->children(); i--; ) {
    Fl_Widget* o = *a++;
    if ((o->x() == (u->x() + u->w()))) { // Shares the right border
      if ((o->y() == u->y()) && (o->h() == u->h())) { // Widgets are the same height.
	dw = o->w() + u->w();
	o->hide();
	t->remove(o);
	//u->w(dw);
	u->size(dw, u->h());
	return o;
      }  
    }
  }
  return 0;
}

GFL_PROCEDURE_2_1
  (fltk_tile_fill_right,
   "fltk-tile::fill-right",
   "If the widget to the right of X is the same height and is horizontally "
   "aligned, it will be removed and X will be expanded to replace it.",
   tile, X,
   fltk_tile, fltk_widget,
   removed,
   fltk_widget)
{
  GFL_ASSERT_2 (fltk_tile_fill_right, tile, X);
  Fl_Widget * r = tile_fill_right ((Fl_Tile *) gh_cdr (tile), (Fl_Widget *) gh_cdr (X));
  return gfl_lookup (r);
}

static Fl_Widget*
tile_fill_left (Fl_Tile *t, Fl_Widget *u)
{
  Fl_Widget*const* a = t->array();
  int dw; // number of pixels to add to u's width.
  int dx; // new x position for u.
  for (int i=t->children(); i--; ) {
    Fl_Widget* o = *a++;
    if ((u->x() == o->x() + o->w())) { // Shares the left border
      if ((o->y() == u->y()) && (o->h() == u->h())) { // Widgets are the same height.
	dw = o->w() + u->w();
	dx = o->x();
	o->hide();
	t->remove(o);
	//u->x(dx);
	u->position(dx, u->y());
	//u->w(dw);
	u->size(dw, u->h());
	return o;
      }  
    }
  }
  return 0;
}

GFL_PROCEDURE_2_1
  (fltk_tile_fill_left,
   "fltk-tile::fill-left",
   "If the widget to the left of X is the same height and is horizontally "
   "aligned, it will be removed and X will be expanded to replace it.",
   tile, X,
   fltk_tile, fltk_widget, 
   removed,
   fltk_widget)
{
  GFL_ASSERT_2 (fltk_tile_fill_left, tile, X);
  Fl_Widget * r = tile_fill_left ((Fl_Tile *) gh_cdr (tile), (Fl_Widget *) gh_cdr (X));
  return gfl_lookup (r);
}

Fl_Widget*
tile_fill_down (Fl_Tile *t, Fl_Widget *u)
{
  Fl_Widget*const* a = t->array();
  int dh = 0; // number of pixels to add to u's height.
  for (int i=t->children(); i--; ) {
    Fl_Widget* o = *a++;
    if ((o->y() == u->y() + u->h())) { // Shares the bottom border
      if ((o->x() == u->x()) && (o->w() == u->w())) { // Widgets are the same width.
	dh = o->h() + u->h();
	o->hide();
	t->remove(o);
	//u->h(dh);
	u->size(u->w(), dh);
	return o;
      }  
    }
  }
  return 0;
}

GFL_PROCEDURE_2_1
  (fltk_tile_fill_down,
   "fltk-tile::fill-down",
   "If the widget below X is the same width and is vertically "
   "aligned, it will be removed and X will be expanded to replace it.",
   tile, X,
   fltk_tile, fltk_widget, 
   removed,
   fltk_widget)
{
  GFL_ASSERT_2 (fltk_tile_fill_down, tile, X);
  Fl_Widget * r = tile_fill_down ((Fl_Tile *) gh_cdr (tile), (Fl_Widget *) gh_cdr (X));
  return gfl_lookup (r);
}

Fl_Widget*
tile_fill_up (Fl_Tile *t, Fl_Widget *u)
{
  Fl_Widget*const* a = t->array();
  int dh; // number of pixels to add to u's height.
  int dy; // new y position for u.
  for (int i=t->children(); i--; ) {
    Fl_Widget* o = *a++;
    if ((u->y() == o->y() + o->h())) { // Shares the top border
      if ((o->x() == u->x()) && (o->w() == u->w())) { // Widgets are the same height.
	dh = o->h() + u->h();
	dy = o->y();
	o->hide();
	t->remove(o);
	//u->y(dy);
	u->position(u->x(), dy);
	//u->h(dh);
	u->size(u->w(), dh);
	return o;
      }  
    }
  }
  return 0;
}

GFL_PROCEDURE_2_1
  (fltk_tile_fill_up,
   "fltk-tile::fill-up",
   "If the widget above X is the same width and is vertically "
   "aligned, it will be removed and X will be expanded to replace it.",
   tile, X,
   fltk_tile, fltk_widget, 
   removed,
   fltk_widget)
{
  GFL_ASSERT_2 (fltk_tile_fill_up, tile, X);
  Fl_Widget * r = tile_fill_up ((Fl_Tile *) gh_cdr (tile), (Fl_Widget *) gh_cdr (X));
  return gfl_lookup (r);
}

/*
   4.  Initialization Hook:
*/

void
fltk_tile_init ()
{
  GFL_NEW_TYPE (fltk_tile);
  GFL_NEW_PROCEDURE (fltk_tile_new);
  GFL_NEW_PROCEDURE (fltk_tile_position);
  GFL_NEW_PROCEDURE (fltk_tile_resize);
  GFL_NEW_PROCEDURE (fltk_tile_fill_left);
  GFL_NEW_PROCEDURE (fltk_tile_fill_right);
  GFL_NEW_PROCEDURE (fltk_tile_fill_up);
  GFL_NEW_PROCEDURE (fltk_tile_fill_down);
  GFL_NEW_PROCEDURE (fltk_tile_insert_left);
  GFL_NEW_PROCEDURE (fltk_tile_insert_right);
  GFL_NEW_PROCEDURE (fltk_tile_insert_up);
  GFL_NEW_PROCEDURE (fltk_tile_insert_down);
}
